from common.PlatformManagement import PlatformManagement
from common.SmartdataClient import SmartdataClient
import time

# Initializing Test Environment
service = PlatformManagement()
domain = 'smartdatatest' + str(int(time.time()))
assert service.createDomain(domain), 'Domain was not created'
cert = service.generateCredentials(domain)
assert cert['serial'], 'Cert was not created'
with open("/tmp/client.key", "w", encoding="utf-8") as file:
    file.write(str(cert['key']))
with open("/tmp/client.crt", "w", encoding="utf-8") as file:
    file.write(str(cert['pem']))

unit = 0x84925924
client = SmartdataClient()
r = 100
t0 = 1
tf = 100
dev = 0
sig1 = 'sig1'

# cria series
serie = client.createSeries(unit, 0, 0, 0, r, t0, tf, dev, sig1)
assert serie, "SERIE deveria ter retornado True"
# get sem retorno
get = client.get(unit, 1, 1, 1, 10, 0, 10, dev, sig1)
assert get['series'] == [], f"GET deveria ter retornado vazio, mas retornou {get}"
# put dado
value = 5
put = client.put(unit, value, 1, 1, 1, 3, dev, sig1)
assert put, f"PUT deveria ter retornado True, mas retornou {put}"
# get com retorno
get = client.get(unit, 1, 1, 1, 10, 0, 10, dev, sig1)
assert get != [], f"GET deveria ter retornado algo, mas retornou {get}"
incoming = get['series'][0]['value']
assert incoming == value, f"GET deveria ter retornado algo, mas retornou {incoming}"
# put dado fora com dev desconhecido
put = client.put(unit, 1, 1, 1, 1, tf - 1, dev + 99, sig1)
assert put == False, f"PUT deveria ter retornado False, mas retornou {put}"
# put dado fora do timerange
put = client.put(unit, 1, 1, 1, 1, tf + 1, dev, sig1)
assert put == False, f"PUT deveria ter retornado False, mas retornou {put}"


#  Testar MV1.2 do put.php
print("### Teste com add/commit (multiplos smartdata v1.2) ")
r = client.commit()
assert r == False, f"Commit zerado deveria retornar False: {r}"
print('Commit zerado: OK')
# sd1 = {"unit": unit, "value": value, "x": 2, "y": 2, "z": 2, "t": 20, "dev": 0, "sig": sig1}
sd1 = [unit, value, 3, 3, 3, t0 + 20, dev, sig1]
sd2 = [unit, value + 1, 3, 3, 3, t0 + 21, dev, sig1]
client.add(*sd1)
client.add(*sd2)
r = client.commit()
assert r, f"Commit deveria ter retornado True: {r}"
print('Commit retornou true: OK')
r = client.get(unit, 1, 1, 1, 10, 20, 30, 0, sig1)
assert r, f"Deveria retornar algo do GET: {r}"
print('Get retornou algo: OK')
assert r['series'], f"Retorno deveria ter serie dentro: {r}"
print('Get retornou uma serie: OK')
assert len(r['series']) == 2, f"Deveria ter dois itens dentro da serie: {r}"
print('Get retornou dois itens: OK')
assert r['series'][0]['value'] == value, f"Valor de retorno nao esta batendo: {r}"
assert r['series'][1]['value'] == value + 1, f"Valor de retorno nao esta batendo: {r}"
print('Valores de retorno estão batendo')
r = client.commit()
assert r == False, f"Commit apos commit deveria voltar False: {r}"
print('Commit depois do commit voltou zerado: OK')


print('### Testando add/commit de multiplos Smartdata de series diferentes')

serie = client.createSeries(unit, 0, 0, 0, r, t0, tf, dev + 1, sig1)
sd1 = [unit, value, 3, 3, 3, t0 + 30, dev, sig1]
sd2 = [unit, value + 1, 3, 3, 3, t0 + 31, dev + 1, sig1]
client.add(*sd1)
client.add(*sd2)
r = client.commit()
assert r, f"Commit deveria ter retornado True: {r}"
print('Commit retornou true: OK')
r = client.get(unit, 1, 1, 1, 50, t0, tf, dev + 1, sig1)
assert r, f"Deveria retornar algo do GET: {r}"
print('Get retornou algo: OK')
assert r['series'], f"Retorno deveria ter serie dentro: {r}"
print('Get retornou uma serie: OK')
assert len(r['series']) == 1, f"Deveria ter 1 itens dentro da serie: {r}"
print('Get retornou 1 item: OK')
assert r['series'][0]['dev'] == dev + 1, f"Dev de retorno nao esta batendo: {r}"
print('Dev de retorno estão batendo')
assert r['series'][0]['value'] == value + 1, f"Value de retorno nao esta batendo: {r}"
print('Valores de retorno estão batendo')


# TODO Testar com v1.1
